"""sageLLM CLI module."""

from __future__ import annotations

from typing import Any


def main(*args: Any, **kwargs: Any) -> Any:
    """Lazy CLI entry point to avoid importing ``sagellm.cli.main`` on package import."""
    from .main import main as _main

    return _main(*args, **kwargs)


def setup_wizard() -> None:
    """Entry point for `sagellm-setup` script."""
    from .commands.setup import setup

    setup.main(prog_name="sagellm-setup", standalone_mode=True)


__all__ = ["main", "setup_wizard"]
