"""Regression tests for CLI backend install configurations."""

from __future__ import annotations

from sagellm.cli.hardware.install_configs import PYTORCH_INSTALL_CONFIGS


def test_ascend_install_config_includes_scipy() -> None:
    packages = PYTORCH_INSTALL_CONFIGS["ascend"]["packages"]

    assert "scipy" in packages


def test_ascend_verify_script_checks_tbe_runtime() -> None:
    verify_script = PYTORCH_INSTALL_CONFIGS["ascend"]["verify_script"]

    assert "importlib.import_module('tbe')" in verify_script
